@@ -44,6 +44,7 @@
 //! }
 //!
 //! assert_eq!(rx.iter().take(n_jobs).fold(0, |a, b| a + b), 8);
+//! # pool.shutdown().expect("shutdown");
 //! ```
 
 use super::*;
@@ -88,6 +89,7 @@ impl ThreadPool {
     /// use executors::crossbeam_channel_pool::ThreadPool;
     ///
     /// let pool = ThreadPool::new(4);
+    /// # pool.shutdown();
     /// ```
     pub fn new(threads: usize) -> ThreadPool {
         assert!(threads > 0);
@@ -115,8 +117,8 @@ impl ThreadPool {
         {
             let cores = core_affinity::get_core_ids().expect("core ids");
             let num_pinned = cores.len().min(threads);
-            for i in 0..num_pinned {
-                spawn_worker_pinned(pool.core.clone(), cores[i]);
+            for core in cores.into_iter().take(num_pinned) {
+                spawn_worker_pinned(pool.core.clone(), core);
             }
             if num_pinned < threads {
                 let num_unpinned = threads - num_pinned;
@@ -145,12 +147,12 @@ impl ThreadPool {
         let pool = ThreadPool {
             core: Arc::new(Mutex::new(ThreadPoolCore {
                 sender: tx.clone(),
-                receiver: rx.clone(),
+                receiver: rx,
                 shutdown: shutdown.clone(),
                 threads: total_threads,
                 ids: 0,
             })),
-            sender: tx.clone(),
+            sender: tx,
             threads: total_threads,
             shutdown,
         };
@@ -163,7 +165,7 @@ impl ThreadPool {
         pool
     }
 
-    fn schedule_task(&self, task: async_task::Task<()>) -> () {
+    fn schedule_task(&self, task: async_task::Runnable) -> () {
         // schedule tasks always, even if the pool is already stopped, since it's unsafe to drop from the schedule function
         // this might lead to some "memory leaks" if an executor remains stopped but allocated for a long time
 
@@ -249,13 +251,9 @@ impl FuturesExecutor for ThreadPool {
         future: impl Future<Output = R> + 'static + Send,
     ) -> JoinHandle<R> {
         let exec = self.clone();
-        let (task, handle) = async_task::spawn(
-            future,
-            move |task| {
-                exec.schedule_task(task);
-            },
-            (),
-        );
+        let (task, handle) = async_task::spawn(future, move |task| {
+            exec.schedule_task(task);
+        });
         task.schedule();
         handle
     }
@@ -267,7 +265,10 @@ fn spawn_worker(core: Arc<Mutex<ThreadPoolCore>>) {
         guard.new_worker_id()
     };
     let mut worker = ThreadPoolWorker::new(id, core);
-    thread::spawn(move || worker.run());
+    thread::Builder::new()
+        .name("cb-channel-pool-worker".to_string())
+        .spawn(move || worker.run())
+        .expect("Could not create worker thread!");
 }
 
 #[cfg(feature = "thread-pinning")]
@@ -276,11 +277,14 @@ fn spawn_worker_pinned(core: Arc<Mutex<ThreadPoolCore>>, core_id: core_affinity:
         let mut guard = core.lock().unwrap();
         guard.new_worker_id()
     };
-    let mut worker = ThreadPoolWorker::new(id, core.clone());
-    thread::spawn(move || {
-        core_affinity::set_for_current(core_id);
-        worker.run()
-    });
+    let mut worker = ThreadPoolWorker::new(id, core);
+    thread::Builder::new()
+        .name("cb-channel-pool-worker".to_string())
+        .spawn(move || {
+            core_affinity::set_for_current(core_id);
+            worker.run()
+        })
+        .expect("Could not create worker thread!");
 }
 
 #[derive(Debug)]
@@ -356,11 +360,8 @@ impl ThreadPoolWorker {
             recv,
         }
     }
-    fn id(&self) -> &usize {
-        &self.id
-    }
     fn run(&mut self) {
-        debug!("CrossbeamWorker {} starting", self.id());
+        debug!("CrossbeamWorker {} starting", self.id);
         let sender = {
             let core = self.core.upgrade().expect("Core already shut down!");
             let guard = core.lock().expect("Mutex poisoned!");
@@ -375,30 +376,20 @@ impl ThreadPoolWorker {
                     let _ = t.run();
                 }
                 JobMsg::Stop(latch) => {
-                    ignore(latch.decrement());
+                    let _ = latch.decrement();
                     break;
                 }
             }
         }
         unset_local_executor();
         sentinel.cancel();
-        debug!("CrossbeamWorker {} shutting down", self.id());
+        debug!("CrossbeamWorker {} shutting down", self.id);
     }
 }
 
-// trait FnBox {
-//     fn call_box(self: Box<Self>);
-// }
-
-// impl<F: FnOnce()> FnBox for F {
-//     fn call_box(self: Box<F>) {
-//         (*self)()
-//     }
-// }
-
 enum JobMsg {
     Job(Box<dyn FnOnce() + Send + 'static>),
-    Task(async_task::Task<()>),
+    Task(async_task::Runnable),
     Stop(Arc<CountdownEvent>),
 }
 
@@ -437,7 +428,6 @@ impl Drop for Sentinel {
 
 #[cfg(test)]
 mod tests {
-    use env_logger;
 
     use super::*;
 
@@ -453,8 +443,7 @@ mod tests {
     #[test]
     fn test_sleepy() {
         let exec = ThreadPool::new(4);
-        crate::tests::test_sleepy(&exec, LABEL);
-        exec.shutdown().expect("Pool didn't shut down!");
+        crate::tests::test_sleepy(exec, LABEL);
     }
 
     #[test]
@@ -468,6 +457,12 @@ mod tests {
         crate::tests::test_local(exec, LABEL);
     }
 
+    #[test]
+    fn test_custom_large() {
+        let exec = ThreadPool::new(36);
+        crate::tests::test_custom(exec, LABEL);
+    }
+
     #[cfg(feature = "thread-pinning")]
     #[test]
     fn test_custom_affinity() {
@@ -486,13 +481,20 @@ mod tests {
         let pool = ThreadPool::new(2);
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(5));
         assert_eq!(res, 0);
+        pool.shutdown().expect("shutdown");
     }
 
+    // replace ignore with panic cfg gate when https://github.com/rust-lang/rust/pull/74754 is merged
     #[test]
+    #[ignore]
     fn keep_pool_size() {
         let _ = env_logger::try_init();
 
@@ -500,11 +502,16 @@ mod tests {
         let pool = ThreadPool::new(1);
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.execute(move || panic!("test panic please ignore"));
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(5));
         assert_eq!(res, 0);
+        pool.shutdown().expect("shutdown");
     }
 
     #[test]
@@ -518,14 +525,18 @@ mod tests {
         let latch3 = latch.clone();
         let stop_latch = Arc::new(CountdownEvent::new(1));
         let stop_latch2 = stop_latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.execute(move || {
             pool2.shutdown_async();
-            ignore(stop_latch2.decrement());
+            let _ = stop_latch2.decrement();
         });
         let res = stop_latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 0);
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 1);
     }
@@ -539,14 +550,20 @@ mod tests {
         let latch = Arc::new(CountdownEvent::new(2));
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.shutdown().expect("pool to shut down");
-        pool2.execute(move || ignore(latch3.decrement()));
+        pool2.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 1);
     }
 
+    // replace ignore with panic cfg gate when https://github.com/rust-lang/rust/pull/74754 is merged
     #[test]
+    #[ignore]
     fn shutdown_on_handle_drop() {
         let _ = env_logger::try_init();
 
