@@ -61,12 +61,19 @@
 //! }
 //!
 //! assert_eq!(rx.iter().take(n_jobs).fold(0, |a, b| a + b), 8);
+//! # pool.shutdown();
 //! ```
 
 use super::*;
 use crate::futures_executor::FuturesExecutor;
+#[cfg(feature = "numa-aware")]
+use crate::numa_utils::{equidistance, ProcessingUnitDistance};
 use crate::parker;
-use crate::parker::{DynParker, ParkResult, Parker};
+#[cfg(not(feature = "ws-no-park"))]
+use crate::parker::ParkResult;
+use crate::parker::{DynParker, Parker};
+#[cfg(feature = "thread-pinning")]
+use core_affinity::CoreId;
 use crossbeam_channel as channel;
 use crossbeam_deque as deque;
 use crossbeam_utils::Backoff;
@@ -127,6 +134,7 @@ pub fn dyn_pool(threads: usize) -> ThreadPool<parker::StaticParker<parker::Dynam
 /// use executors::crossbeam_workstealing_pool;
 ///
 /// let pool = crossbeam_workstealing_pool::pool_with_auto_parker(4);
+/// # pool.shutdown();
 /// ```
 pub fn pool_with_auto_parker(threads: usize) -> ThreadPool<DynParker> {
     if threads <= parker::SmallThreadData::MAX_THREADS {
@@ -192,6 +200,8 @@ where
     threads: usize,
     shutdown: AtomicBool,
     parker: P,
+    #[cfg(feature = "numa-aware")]
+    pu_distance: ProcessingUnitDistance,
 }
 
 #[derive(Debug)]
@@ -210,8 +220,8 @@ where
     ///
     /// # Panics
     ///
-    /// This function will panic if `threads` is 0.
-    /// It will also panic if `threads` is larger than the `ThreadData::MAX_THREADS` value of the provided `parker`.
+    /// - This function will panic if `threads` is 0.
+    /// - It will also panic if `threads` is larger than the `ThreadData::MAX_THREADS` value of the provided `parker`.
     ///
     /// # Core Affinity
     ///
@@ -228,6 +238,7 @@ where
     /// use executors::crossbeam_workstealing_pool::ThreadPool;
     ///
     /// let pool = ThreadPool::new(4, parker::small());
+    /// # pool.shutdown();
     /// ```
     pub fn new(threads: usize, parker: P) -> ThreadPool<P> {
         assert!(threads > 0);
@@ -235,12 +246,19 @@ where
             assert!(threads <= max);
         }
         let core = ThreadPoolCore::new();
+        #[cfg(feature = "numa-aware")]
+        let pu_distance = {
+            let num_pus = core_affinity::get_core_ids().expect("CoreIds").len(); // this should work, if not we'd fail below anyway
+            ProcessingUnitDistance::from_function(num_pus, equidistance)
+        };
         let inner = ThreadPoolInner {
             core: Mutex::new(core),
             global_sender: deque::Injector::new(),
             threads,
             shutdown: AtomicBool::new(false),
             parker,
+            #[cfg(feature = "numa-aware")]
+            pu_distance,
         };
         let pool = ThreadPool {
             inner: Arc::new(inner),
@@ -258,9 +276,9 @@ where
             let mut guard = pool.inner.core.lock().unwrap();
             let cores = core_affinity::get_core_ids().expect("core ids");
             let num_pinned = cores.len().min(threads);
-            for i in 0..num_pinned {
+            for core in cores.iter().take(num_pinned) {
                 pool.inner
-                    .spawn_worker_pinned(&mut guard, pool.inner.clone(), None, cores[i]);
+                    .spawn_worker_pinned(&mut guard, pool.inner.clone(), None, *core);
             }
             if num_pinned < threads {
                 let num_unpinned = threads - num_pinned;
@@ -280,25 +298,30 @@ where
     ///
     /// # Panics
     ///
-    /// This function will panic if `cores.len() + floating` is 0.
+    /// - This function will panic if `cores.len() + floating` is 0.
+    /// - This function will panic if `cores.len() + floating` is greater than `parker.max_threads()`.
+    /// - This function will panic if no core ids can be accessed.
     #[cfg(feature = "thread-pinning")]
-    pub fn with_affinity(
-        cores: &[core_affinity::CoreId],
-        floating: usize,
-        parker: P,
-    ) -> ThreadPool<P> {
+    pub fn with_affinity(cores: &[CoreId], floating: usize, parker: P) -> ThreadPool<P> {
         let total_threads = cores.len() + floating;
         assert!(total_threads > 0);
         if let Some(max) = parker.max_threads() {
             assert!(total_threads <= max);
         }
         let core = ThreadPoolCore::new();
+        #[cfg(feature = "numa-aware")]
+        let pu_distance = {
+            let num_pus = core_affinity::get_core_ids().expect("CoreIds").len(); // this should work if cores is supplied
+            ProcessingUnitDistance::from_function(num_pus, equidistance)
+        };
         let inner = ThreadPoolInner {
             core: Mutex::new(core),
             global_sender: deque::Injector::new(),
             threads: total_threads,
             shutdown: AtomicBool::new(false),
             parker,
+            #[cfg(feature = "numa-aware")]
+            pu_distance,
         };
         let pool = ThreadPool {
             inner: Arc::new(inner),
@@ -317,7 +340,58 @@ where
         pool
     }
 
-    fn schedule_task(&self, task: async_task::Task<()>) -> () {
+    /// Creates a new thread pool capable of executing `threads` number of jobs concurrently with a particular core affinity.
+    ///
+    /// For each core id in the `core` slice, it will generate a single thread pinned to that id.
+    /// Additionally, it will create `floating` number of unpinned threads.
+    ///
+    /// Internally the stealers will use the provided PU distance matrix to prioritise stealing from
+    /// queues that are "closer" by, in order to try and reduce memory movement across caches and NUMA nodes.
+    ///
+    /// # Panics
+    ///
+    /// - This function will panic if `cores.len() + floating` is 0.
+    /// - This function will panic if `cores.len() + floating` is greater than `parker.max_threads()`.
+    /// - This function will panic if no core ids can be accessed.
+    #[cfg(feature = "numa-aware")]
+    pub fn with_numa_affinity(
+        cores: &[CoreId],
+        floating: usize,
+        parker: P,
+        pu_distance: ProcessingUnitDistance,
+    ) -> ThreadPool<P> {
+        let total_threads = cores.len() + floating;
+        assert!(total_threads > 0);
+        if let Some(max) = parker.max_threads() {
+            assert!(total_threads <= max);
+        }
+        let core = ThreadPoolCore::new();
+        let inner = ThreadPoolInner {
+            core: Mutex::new(core),
+            global_sender: deque::Injector::new(),
+            threads: total_threads,
+            shutdown: AtomicBool::new(false),
+            parker,
+            pu_distance,
+        };
+        let pool = ThreadPool {
+            inner: Arc::new(inner),
+        };
+        {
+            let mut guard = pool.inner.core.lock().unwrap();
+            cores.iter().for_each(|core_id| {
+                pool.inner
+                    .spawn_worker_pinned(&mut guard, pool.inner.clone(), None, *core_id);
+            });
+            for _ in 0..floating {
+                pool.inner
+                    .spawn_worker(&mut guard, pool.inner.clone(), None);
+            }
+        }
+        pool
+    }
+
+    fn schedule_task(&self, task: async_task::Runnable) -> () {
         // schedule tasks always, even if the pool is already stopped, since it's unsafe to drop from the schedule function
         // this might lead to some "memory leaks" if an executor remains stopped but allocated for a long time
         LOCAL_JOB_QUEUE.with(|qo| unsafe {
@@ -432,13 +506,9 @@ where
         future: impl Future<Output = R> + 'static + Send,
     ) -> JoinHandle<R> {
         let exec = self.clone();
-        let (task, handle) = async_task::spawn(
-            future,
-            move |task| {
-                exec.schedule_task(task);
-            },
-            (),
-        );
+        let (task, handle) = async_task::spawn(future, move |task| {
+            exec.schedule_task(task);
+        });
         task.schedule();
         handle
     }
@@ -466,6 +536,9 @@ where
         thread::Builder::new()
             .name(format!("cb-ws-pool-worker-{}", id))
             .spawn(move || {
+                #[cfg(feature = "thread-pinning")]
+                let mut worker = ThreadPoolWorker::new(id, rx_control, &inner, None);
+                #[cfg(not(feature = "thread-pinning"))]
                 let mut worker = ThreadPoolWorker::new(id, rx_control, &inner);
                 drop(inner);
                 worker.run()
@@ -479,7 +552,7 @@ where
         guard: &mut std::sync::MutexGuard<'_, ThreadPoolCore>,
         inner: Arc<Self>,
         old_id: Option<usize>,
-        core_id: core_affinity::CoreId,
+        core_id: CoreId,
     ) {
         //let guard = self.core.lock().unwrap();
         let id = old_id.unwrap_or_else(|| guard.new_worker_id());
@@ -494,7 +567,7 @@ where
             .name(format!("cb-ws-pool-worker-{}", id))
             .spawn(move || {
                 core_affinity::set_for_current(core_id);
-                let mut worker = ThreadPoolWorker::new(id, rx_control, &inner);
+                let mut worker = ThreadPoolWorker::new(id, rx_control, &inner, Some(core_id));
                 drop(inner);
                 worker.run()
             })
@@ -601,8 +674,10 @@ where
     id: usize,
     core: Weak<ThreadPoolInner<P>>,
     control: channel::Receiver<ControlMsg>,
-    stealers: Vec<JobStealer>,
+    stealers: Vec<(i32, JobStealer)>,
     random: ThreadRng,
+    #[cfg(feature = "thread-pinning")]
+    core_id: Option<CoreId>,
 }
 
 impl<P> ThreadPoolWorker<P>
@@ -613,6 +688,7 @@ where
         id: usize,
         control: channel::Receiver<ControlMsg>,
         core: &Arc<ThreadPoolInner<P>>,
+        #[cfg(feature = "thread-pinning")] core_id: Option<CoreId>,
     ) -> ThreadPoolWorker<P> {
         ThreadPoolWorker {
             id,
@@ -620,6 +696,8 @@ where
             control,
             stealers: Vec::new(),
             random: rand::thread_rng(),
+            #[cfg(feature = "thread-pinning")]
+            core_id,
         }
     }
 
@@ -629,26 +707,39 @@ where
     }
 
     #[inline(always)]
+    #[cfg(feature = "ws-no-park")]
+    fn abort_sleep(
+        &self,
+        backoff: &Backoff,
+        snoozing: &mut bool,
+        failed_steal_attempts: &mut i32,
+    ) -> () {
+        if *snoozing {
+            backoff.reset();
+            *snoozing = false;
+        }
+        *failed_steal_attempts = 0;
+    }
+
+    #[inline(always)]
+    #[cfg(not(feature = "ws-no-park"))]
     fn abort_sleep(
         &self,
         core: &Arc<ThreadPoolInner<P>>,
         backoff: &Backoff,
         snoozing: &mut bool,
         parking: &mut bool,
+        failed_steal_attempts: &mut i32,
     ) -> () {
         if *parking {
-            #[cfg(feature = "ws-no-park")]
-            unreachable!("parking should never be true in ws-no-park!");
-            #[cfg(not(feature = "ws-no-park"))]
-            {
-                core.parker.abort_park(*self.id());
-                *parking = false;
-            }
+            core.parker.abort_park(*self.id());
+            *parking = false;
         }
         if *snoozing {
             backoff.reset();
             *snoozing = false;
         }
+        *failed_steal_attempts = 0;
     }
 
     #[inline(always)]
@@ -670,6 +761,10 @@ where
         set_local_executor(ThreadLocalExecute);
         let core = self.core.upgrade().expect("Core shut down already!");
         //info!("Setting up new worker {}. strong={}, weak={}", self.id(), Arc::strong_count(&core), Arc::weak_count(&core));
+
+        #[cfg(feature = "numa-aware")]
+        let local_stealer = JobStealer::new(local_stealer_raw, self.id, self.core_id);
+        #[cfg(not(feature = "numa-aware"))]
         let local_stealer = JobStealer::new(local_stealer_raw, self.id);
         self.register_stealer(&core, local_stealer);
         core.parker.init(*self.id());
@@ -677,7 +772,9 @@ where
         let sentinel = Sentinel::new(self.core.clone(), self.id);
         let backoff = Backoff::new();
         let mut snoozing = false;
+        #[cfg(not(feature = "ws-no-park"))]
         let mut parking = false;
+        let mut failed_steal_attempts: i32 = 0;
         let mut stop_latch: Option<Arc<CountdownEvent>>;
         'main: loop {
             //info!("Worker {} starting main loop", self.id());
@@ -725,8 +822,21 @@ where
                 match self.control.try_recv() {
                     Ok(msg) => match msg {
                         ControlMsg::Stealers(l) => {
+                            #[cfg(feature = "numa-aware")]
+                            self.update_stealers(&core, l);
+                            #[cfg(not(feature = "numa-aware"))]
                             self.update_stealers(l);
-                            self.abort_sleep(&core, &backoff, &mut snoozing, &mut parking);
+
+                            #[cfg(feature = "ws-no-park")]
+                            self.abort_sleep(&backoff, &mut snoozing, &mut failed_steal_attempts);
+                            #[cfg(not(feature = "ws-no-park"))]
+                            self.abort_sleep(
+                                &core,
+                                &backoff,
+                                &mut snoozing,
+                                &mut parking,
+                                &mut failed_steal_attempts,
+                            );
                         }
                         ControlMsg::Stop(latch) => {
                             stop_latch = Some(latch);
@@ -752,7 +862,16 @@ where
             });
             if let deque::Steal::Success(msg) = glob_res {
                 msg.run();
-                self.abort_sleep(&core, &backoff, &mut snoozing, &mut parking);
+                #[cfg(feature = "ws-no-park")]
+                self.abort_sleep(&backoff, &mut snoozing, &mut failed_steal_attempts);
+                #[cfg(not(feature = "ws-no-park"))]
+                self.abort_sleep(
+                    &core,
+                    &backoff,
+                    &mut snoozing,
+                    &mut parking,
+                    &mut failed_steal_attempts,
+                );
                 continue 'main;
             }
             // only go on if there was no work left on the local queue
@@ -762,19 +881,36 @@ where
                 continue 'main;
             }
             // try to steal something!
-            for stealer in self.stealers.iter() {
+            for (_weight, stealer) in self
+                .stealers
+                .iter()
+                .take_while(|(weight, _)| *weight <= failed_steal_attempts)
+            {
                 if let deque::Steal::Success(msg) = stealer.steal() {
                     msg.run();
-                    self.abort_sleep(&core, &backoff, &mut snoozing, &mut parking);
+                    #[cfg(feature = "ws-no-park")]
+                    self.abort_sleep(&backoff, &mut snoozing, &mut failed_steal_attempts);
+                    #[cfg(not(feature = "ws-no-park"))]
+                    self.abort_sleep(
+                        &core,
+                        &backoff,
+                        &mut snoozing,
+                        &mut parking,
+                        &mut failed_steal_attempts,
+                    );
                     continue 'main; // only steal once before checking locally again
                 }
             }
+            failed_steal_attempts = failed_steal_attempts.saturating_add(1);
             snoozing = true;
-            if parking {
-                #[cfg(feature = "ws-no-park")]
-                unreachable!("parking should never be true in ws-no-park!");
-                #[cfg(not(feature = "ws-no-park"))]
-                {
+            #[cfg(feature = "ws-no-park")]
+            {
+                drop(core);
+                backoff.snooze();
+            }
+            #[cfg(not(feature = "ws-no-park"))]
+            {
+                if parking {
                     let parker = core.parker.clone();
                     drop(core);
                     match parker.park(*self.id()) {
@@ -783,13 +919,13 @@ where
                             self.stop_sleep(&backoff, &mut snoozing, &mut parking);
                         }
                     }
+                } else if backoff.is_completed() {
+                    core.parker.prepare_park(*self.id());
+                    parking = true;
+                } else {
+                    drop(core);
+                    backoff.snooze();
                 }
-            } else if backoff.is_completed() && cfg!(not(feature = "ws-no-park")) {
-                core.parker.prepare_park(*self.id());
-                parking = true;
-            } else {
-                drop(core);
-                backoff.snooze();
             }
             // aaaaand starting over with 'local
         }
@@ -817,35 +953,66 @@ where
         guard.drop_worker(self.id);
     }
 
-    fn update_stealers(&mut self, mut v: Vec<JobStealer>) {
-        v.shuffle(&mut self.random);
-        self.stealers = v;
+    #[cfg(not(feature = "numa-aware"))]
+    fn update_stealers(&mut self, v: Vec<JobStealer>) {
+        let mut weighted: Vec<(i32, JobStealer)> =
+            v.into_iter().map(|stealer| (0, stealer)).collect();
+        weighted.shuffle(&mut self.random);
+        self.stealers = weighted;
+    }
+
+    #[cfg(feature = "numa-aware")]
+    fn update_stealers(&mut self, _core: &ThreadPoolInner<P>, v: Vec<JobStealer>) {
+        let mut weighted: Vec<(i32, JobStealer)> = if let Some(my_id) = self.core_id {
+            let distances = &_core.pu_distance;
+            v.into_iter()
+                .map(|stealer| {
+                    let weight = if let Some(stealer_id) = stealer.core_id() {
+                        distances.distance(my_id, *stealer_id)
+                    } else {
+                        std::i32::MIN
+                    };
+                    (weight, stealer)
+                })
+                .collect()
+        } else {
+            v.into_iter().map(|stealer| (0, stealer)).collect()
+        };
+        weighted.shuffle(&mut self.random);
+        weighted.sort_unstable_by_key(|(weight, _stealer)| *weight);
+        self.stealers = weighted;
     }
 }
 
-// trait FnBox {
-//     fn call_box(self: Box<Self>);
-// }
-
-// impl<F: FnOnce()> FnBox for F {
-//     fn call_box(self: Box<F>) {
-//         (*self)()
-//     }
-// }
-
 #[derive(Clone)]
 struct JobStealer {
     inner: deque::Stealer<Job>,
     id: usize,
+    #[cfg(feature = "numa-aware")]
+    core_id: Option<CoreId>,
 }
 
 impl JobStealer {
-    fn new(stealer: deque::Stealer<Job>, id: usize) -> JobStealer {
-        JobStealer { inner: stealer, id }
+    fn new(
+        stealer: deque::Stealer<Job>,
+        id: usize,
+        #[cfg(feature = "numa-aware")] core_id: Option<CoreId>,
+    ) -> JobStealer {
+        JobStealer {
+            inner: stealer,
+            id,
+            #[cfg(feature = "numa-aware")]
+            core_id,
+        }
     }
     fn id(&self) -> usize {
         self.id
     }
+
+    #[cfg(feature = "numa-aware")]
+    fn core_id(&self) -> &Option<CoreId> {
+        &self.core_id
+    }
 }
 
 impl PartialEq for JobStealer {
@@ -872,7 +1039,7 @@ impl Debug for JobStealer {
 //struct Job(Box<dyn FnOnce() + Send + 'static>);
 enum Job {
     Function(Box<dyn FnOnce() + Send + 'static>),
-    Task(async_task::Task<()>),
+    Task(async_task::Runnable),
 }
 impl Job {
     fn run(self) {
@@ -966,11 +1133,10 @@ where
 
 #[cfg(test)]
 mod tests {
-    use env_logger;
 
     use super::*;
 
-    const LABEL: &'static str = "Workstealing Pool";
+    const LABEL: &str = "Workstealing Pool";
 
     #[test]
     fn test_debug() {
@@ -980,27 +1146,50 @@ mod tests {
     }
 
     #[test]
-    fn test_sleepy() {
+    fn test_sleepy_small() {
         let exec = ThreadPool::new(4, parker::small());
-        crate::tests::test_sleepy(&exec, LABEL);
-        exec.shutdown().expect("Pool didn't shut down!");
+        crate::tests::test_sleepy(exec, LABEL);
+    }
+
+    #[test]
+    fn test_sleepy_large() {
+        let exec = ThreadPool::new(36, parker::large());
+        crate::tests::test_sleepy(exec, LABEL);
+    }
+
+    #[test]
+    fn test_sleepy_dyn() {
+        let exec = ThreadPool::new(72, parker::dynamic());
+        crate::tests::test_sleepy(exec, LABEL);
     }
 
     #[test]
     fn test_custom_small() {
-        let exec = ThreadPool::new(4, parker::small());
+        let exec = small_pool(4);
         crate::tests::test_custom(exec, LABEL);
     }
 
     #[test]
     fn test_custom_large() {
-        let exec = ThreadPool::new(36, parker::large());
+        let exec = large_pool(36);
         crate::tests::test_custom(exec, LABEL);
     }
 
     #[test]
     fn test_custom_dyn() {
-        let exec = ThreadPool::new(72, parker::dynamic());
+        let exec = dyn_pool(72);
+        crate::tests::test_custom(exec, LABEL);
+    }
+
+    #[test]
+    fn test_custom_auto_large() {
+        let exec = pool_with_auto_parker(36);
+        crate::tests::test_custom(exec, LABEL);
+    }
+
+    #[test]
+    fn test_custom_auto_dyn() {
+        let exec = pool_with_auto_parker(72);
         crate::tests::test_custom(exec, LABEL);
     }
 
@@ -1020,6 +1209,17 @@ mod tests {
         crate::tests::test_custom(exec, LABEL);
     }
 
+    #[cfg(feature = "numa-aware")]
+    #[test]
+    fn test_custom_numa_affinity() {
+        let cores = core_affinity::get_core_ids().expect("core ids");
+        // travis doesn't have enough cores for this
+        //let exec = ThreadPool::with_affinity(&cores[0..4], 4, parker::small());
+        let distances = ProcessingUnitDistance::from_function(cores.len(), equidistance);
+        let exec = ThreadPool::with_numa_affinity(&cores[0..1], 1, parker::small(), distances);
+        crate::tests::test_custom(exec, LABEL);
+    }
+
     #[test]
     fn test_defaults() {
         crate::tests::test_defaults::<ThreadPool<DynParker>>(LABEL);
@@ -1033,8 +1233,12 @@ mod tests {
         let pool = ThreadPool::new(2, parker::small());
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(5));
         assert_eq!(res, 0);
         pool.shutdown()
@@ -1052,12 +1256,16 @@ mod tests {
         let latch2 = latch.clone();
         let latch3 = latch.clone();
         pool.execute(move || {
-            ignore(latch2.decrement());
-            pool2.execute(move || ignore(latch2.decrement()));
+            let _ = latch2.decrement();
+            pool2.execute(move || {
+                let _ = latch2.decrement();
+            });
         });
         pool.execute(move || {
-            ignore(latch3.decrement());
-            pool3.execute(move || ignore(latch3.decrement()));
+            let _ = latch3.decrement();
+            pool3.execute(move || {
+                let _ = latch3.decrement();
+            });
         });
         let res = latch.wait_timeout(Duration::from_secs(5));
         assert_eq!(res, 0);
@@ -1065,7 +1273,9 @@ mod tests {
             .unwrap_or_else(|e| error!("Error during pool shutdown {:?}", e));
     }
 
+    // replace ignore with panic cfg gate when https://github.com/rust-lang/rust/pull/74754 is merged
     #[test]
+    #[ignore]
     fn keep_pool_size() {
         let _ = env_logger::try_init();
 
@@ -1073,16 +1283,22 @@ mod tests {
         let pool = ThreadPool::new(1, parker::small());
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.execute(move || panic!("test panic please ignore"));
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(5));
         assert_eq!(res, 0);
         pool.shutdown()
             .unwrap_or_else(|e| error!("Error during pool shutdown {:?}", e));
     }
 
+    // replace ignore with panic cfg gate when https://github.com/rust-lang/rust/pull/74754 is merged
     #[test]
+    #[ignore]
     fn reassign_jobs_from_failed_queues() {
         let _ = env_logger::try_init();
 
@@ -1091,9 +1307,13 @@ mod tests {
         let pool2 = pool.clone();
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
         pool.execute(move || {
-            pool2.execute(move || ignore(latch3.decrement()));
+            let _ = latch2.decrement();
+        });
+        pool.execute(move || {
+            pool2.execute(move || {
+                let _ = latch3.decrement();
+            });
             panic!("test panic please ignore")
         });
         let res = latch.wait_timeout(Duration::from_secs(5));
@@ -1132,15 +1352,19 @@ mod tests {
         let latch3 = latch.clone();
         let stop_latch = Arc::new(CountdownEvent::new(1));
         let stop_latch2 = stop_latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.execute(move || {
             pool2.shutdown_async();
-            ignore(stop_latch2.decrement());
+            let _ = stop_latch2.decrement();
         });
         let res = stop_latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 0);
         thread::sleep(Duration::from_secs(1)); // give the new shutdown thread some time to run
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 1);
     }
@@ -1155,18 +1379,24 @@ mod tests {
         let stop_latch = Arc::new(CountdownEvent::new(1));
         let run_latch2 = run_latch.clone();
         let stop_latch2 = stop_latch.clone();
-        pool.execute(move || ignore(run_latch2.decrement()));
+        pool.execute(move || {
+            let _ = run_latch2.decrement();
+        });
         let res = run_latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 0);
         pool.shutdown().expect("pool to shut down");
-        pool2.execute(move || ignore(stop_latch2.decrement()));
+        pool2.execute(move || {
+            let _ = stop_latch2.decrement();
+        });
         let res = stop_latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 1);
     }
 
     /// This test may/should panic on the worker threads, for example with "Threadpool wasn't shut down properly!".
     /// Otherwise they aren't shut down properly.
+    // replace ignore with panic cfg gate when https://github.com/rust-lang/rust/pull/74754 is merged
     #[test]
+    #[ignore]
     fn dealloc_on_handle_drop() {
         let _ = env_logger::try_init();
 
