@@ -5,7 +5,7 @@
 // <LICENSE or http://opensource.org/licenses/MIT>.
 // This file may not be copied, modified, or distributed
 // except according to those terms.
-#![doc(html_root_url = "https://docs.rs/executors/0.7.0")]
+#![doc(html_root_url = "https://docs.rs/executors/0.8.0")]
 #![deny(missing_docs)]
 #![allow(unused_parens)]
 #![allow(clippy::unused_unit)]
@@ -25,13 +25,14 @@ pub mod common;
 pub mod crossbeam_channel_pool;
 #[cfg(feature = "workstealing-exec")]
 pub mod crossbeam_workstealing_pool;
+#[cfg(feature = "numa-aware")]
+pub mod numa_utils;
 pub mod parker;
 pub mod run_now;
 #[cfg(feature = "threadpool-exec")]
 pub mod threadpool_executor;
 mod timeconstants;
 
-use crate::common::ignore;
 pub use crate::common::{CanExecute, Executor};
 
 #[cfg(any(feature = "cb-channel-exec", feature = "workstealing-exec"))]
@@ -53,9 +54,10 @@ pub(crate) mod tests {
     use std::sync::Arc;
     use std::time::Duration;
 
-    pub const N_DEPTH: usize = 4096;
-    //pub const N_DEPTH: usize = 100000; // run_now can't do this, but it's a good test for the others
+    pub const N_DEPTH_SMALL: usize = 1024;
+    pub const N_DEPTH: usize = 8192; // run_now can't do this, but it's a good test for the others
     pub const N_WIDTH: usize = 128;
+    pub const N_SLEEP_ROUNDS: usize = 11;
 
     pub fn test_debug<E>(exec: &E, label: &str)
     where
@@ -64,6 +66,26 @@ pub(crate) mod tests {
         println!("Debug output for {}: {:?}", label, exec);
     }
 
+    pub fn test_small_defaults<E>(label: &str)
+    where
+        E: Executor + Debug + std::default::Default + 'static,
+    {
+        let pool = E::default();
+
+        let latch = Arc::new(CountdownEvent::new(N_DEPTH_SMALL * N_WIDTH));
+        for _ in 0..N_WIDTH {
+            let pool2 = pool.clone();
+            let latch2 = latch.clone();
+            pool.execute(move || {
+                do_step(latch2, pool2, N_DEPTH_SMALL);
+            });
+        }
+        let res = latch.wait_timeout(Duration::from_secs(30));
+        assert_eq!(res, 0);
+        pool.shutdown()
+            .unwrap_or_else(|e| error!("Error during pool shutdown {:?} at {}", e, label));
+    }
+
     pub fn test_defaults<E>(label: &str)
     where
         E: Executor + Debug + std::default::Default + 'static,
@@ -78,7 +100,7 @@ pub(crate) mod tests {
                 do_step(latch2, pool2, N_DEPTH);
             });
         }
-        let res = latch.wait_timeout(Duration::from_secs(30));
+        let res = latch.wait_timeout(Duration::from_secs(60));
         assert_eq!(res, 0);
         pool.shutdown()
             .unwrap_or_else(|e| error!("Error during pool shutdown {:?} at {}", e, label));
@@ -104,27 +126,27 @@ pub(crate) mod tests {
             .unwrap_or_else(|e| error!("Error during pool shutdown {:?} at {}", e, label));
     }
 
-    pub fn test_sleepy<E>(pool: &E, label: &str)
+    pub fn test_sleepy<E>(pool: E, label: &str)
     where
         E: Executor + 'static,
     {
         info!("Running sleepy test for {}", label);
-        let latch = Arc::new(CountdownEvent::new(2 * N_WIDTH));
-        for _ in 0..N_WIDTH {
-            let latch2 = latch.clone();
-            pool.execute(move || {
-                latch2.decrement().expect("Latch didn't decrement!");
-            });
+        let latch = Arc::new(CountdownEvent::new(N_SLEEP_ROUNDS * N_WIDTH));
+        for round in 1..=N_SLEEP_ROUNDS {
+            // let threads go to sleep
+            let sleep_time = 1u64 << round;
+            std::thread::sleep(Duration::from_millis(sleep_time));
+            for _ in 0..N_WIDTH {
+                let latch2 = latch.clone();
+                pool.execute(move || {
+                    latch2.decrement().expect("Latch didn't decrement!");
+                });
+            }
         }
-        std::thread::sleep(Duration::from_secs(1));
-        for _ in 0..N_WIDTH {
-            let latch2 = latch.clone();
-            pool.execute(move || {
-                latch2.decrement().expect("Latch didn't decrement!");
-            });
-        }
-        let res = latch.wait_timeout(Duration::from_secs(5));
+        let res = latch.wait_timeout(Duration::from_secs((N_SLEEP_ROUNDS as u64) * 3));
         assert_eq!(res, 0);
+        pool.shutdown()
+            .unwrap_or_else(|e| error!("Error during pool shutdown {:?} at {}", e, label));
     }
 
     pub fn test_local<E>(exec: E, label: &str)
@@ -152,6 +174,31 @@ pub(crate) mod tests {
             .unwrap_or_else(|e| error!("Error during pool shutdown {:?} at {}", e, label));
     }
 
+    pub fn test_small_local<E>(exec: E, label: &str)
+    where
+        E: Executor + Debug + 'static,
+    {
+        let pool = exec;
+
+        let latch = Arc::new(CountdownEvent::new(N_DEPTH_SMALL * N_WIDTH));
+        let failed = Arc::new(AtomicBool::new(false));
+        for _ in 0..N_WIDTH {
+            let latch2 = latch.clone();
+            let failed2 = failed.clone();
+            pool.execute(move || {
+                do_step_local(latch2, failed2, N_DEPTH_SMALL);
+            });
+        }
+        let res = latch.wait_timeout(Duration::from_secs(30));
+        assert_eq!(res, 0);
+        assert!(
+            !failed.load(Ordering::SeqCst),
+            "Executor does not support local scheduling!"
+        );
+        pool.shutdown()
+            .unwrap_or_else(|e| error!("Error during pool shutdown {:?} at {}", e, label));
+    }
+
     fn do_step<E>(latch: Arc<CountdownEvent>, pool: E, depth: usize)
     where
         E: Executor + Debug + 'static,
