@@ -149,8 +149,7 @@ mod tests {
     #[test]
     fn test_sleepy() {
         let exec = ThreadPoolExecutor::new(4);
-        crate::tests::test_sleepy(&exec, LABEL);
-        exec.shutdown().expect("Pool didn't shut down!");
+        crate::tests::test_sleepy(exec, LABEL);
     }
 
     #[test]
@@ -158,7 +157,9 @@ mod tests {
         crate::tests::test_defaults::<ThreadPoolExecutor>(LABEL);
     }
 
+    // replace ignore with panic cfg gate when https://github.com/rust-lang/rust/pull/74754 is merged
     #[test]
+    #[ignore]
     #[should_panic] // this executor does not actually support local scheduling
     fn test_local() {
         let exec = ThreadPoolExecutor::default();
@@ -173,13 +174,19 @@ mod tests {
         let pool = ThreadPoolExecutor::new(2);
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(5));
         assert_eq!(res, 0);
     }
 
+    // replace ignore with panic cfg gate when https://github.com/rust-lang/rust/pull/74754 is merged
     #[test]
+    #[ignore]
     fn keep_pool_size() {
         let _ = env_logger::try_init();
 
@@ -187,9 +194,13 @@ mod tests {
         let pool = ThreadPoolExecutor::new(1);
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.execute(move || panic!("test panic please ignore"));
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(5));
         assert_eq!(res, 0);
     }
@@ -205,14 +216,18 @@ mod tests {
         let latch3 = latch.clone();
         let stop_latch = Arc::new(CountdownEvent::new(1));
         let stop_latch2 = stop_latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.execute(move || {
             pool2.shutdown_async();
-            ignore(stop_latch2.decrement());
+            let _ = stop_latch2.decrement();
         });
         let res = stop_latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 0);
-        pool.execute(move || ignore(latch3.decrement()));
+        pool.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 1);
     }
@@ -226,9 +241,13 @@ mod tests {
         let latch = Arc::new(CountdownEvent::new(2));
         let latch2 = latch.clone();
         let latch3 = latch.clone();
-        pool.execute(move || ignore(latch2.decrement()));
+        pool.execute(move || {
+            let _ = latch2.decrement();
+        });
         pool.shutdown().expect("pool to shut down");
-        pool2.execute(move || ignore(latch3.decrement()));
+        pool2.execute(move || {
+            let _ = latch3.decrement();
+        });
         let res = latch.wait_timeout(Duration::from_secs(1));
         assert_eq!(res, 1);
     }
